import os

class Workspace:
    '''
    Docker workspaces are unique in our world, as they can be one of two context
    dependent things: A Docker build directory, containing only a single
    Dockerfile, or they can be part of a formation using docker-compose in which
    they warehouse a docker-compose.yml file.

    Under most situations we only care about the context the charm author wishes
    to be in, and what implications that has on the workspace to be valid.

    This method simply exposes an overrideable object to determine these
    characteristics.
    '''
    def __init__(self, path, context="compose"):
        self.path = path
        self.context = context

    def __str__(self):
        return self.path

    def __repr__(self):
        return self.path

    def validate(self):
        dcyml = os.path.isfile("{}/docker-compose.yml".format(self.path))
        dcyaml = os.path.isfile("{}/docker-compose.yaml".format(self.path))
        dfile = os.path.isfile("{}/Dockerfile".format(self.path))

        if self.context == "compose":
            if not dcyml and not dcyaml:
                msg = "Missing yaml definition: docker-compose.yml"
                raise OSError(msg)
        else:
            if not dfile:
                msg = "Missing Dockerfile"
                raise OSError(msg)
        return True



import os
import subprocess

from contextlib import contextmanager
from shlex import split

from workspace import Workspace

# Wrapper and convenience methods for charming w/ docker in python
class Docker:
    '''
    Wrapper class to communicate with the Docker daemon on behalf of
    a charmer. Provides stateless operations of a running docker daemon
    '''

    def __init__(self, socket="unix:///var/run/docker.sock", workspace=None):
        '''
        @param socket - URI to the Docker daemon socket
            default: unix://var/run/docker.sock

        @param workspace - Path to directory containing a Dockerfile
            default: None
        '''
        self.socket = socket
        if workspace:
            self.workspace = Workspace(workspace)

    def running(self):
        '''
        Predicate method to determine if the daemon we are talking to is
        actually online and recieving events.

        ex: bootstrap = Docker(socket="unix://var/run/docker-boostrap.sock")
        bootstrap.running()
        > True
        '''
        # TODO: Add TCP:// support for running check
        return os.path.isfile(self.socket)

    def run(self, image, options="", volumes=[], ports=[]):
        volumes = " --volume=".join(volumes)
        ports = " -p".join(ports)
        cmd = "docker run {opts} {vols} {ports} {image}".format(
            opts=options, vols=volumes, ports=ports, image=image)

        try:
            subprocess.check_output(split(cmd))
        except subprocess.CalledProcessError as expec:
            print "Error: ", expect.returncode, expect.output



from charmhelpers.core import unitdata


class DockerOpts:

    ''' DockerOptsManager - A Python class for managing the DEFAULT docker
    options on a daemon dynamically. As a docker daemon integrates with more
    services it becomes quickly unweidly to just "template and go" for this
    solution. Having a data bag to stuff in options/multioptions and render to
    a template is a far nicer solution.

    THe underlying data-provider is backed by a SQLITE database on each unit,
    tracking the dictionary, provided from the 'charmhelpers' python package.

    Summary:
    opts = DockerOpts()
    opts.add('mtu', flannel_mtu)
    opts.to_s()
    '''

    def __init__(self, opts_path=None):
        self.db = unitdata.kv()
        if not self.db.get('docker_opts'):
            self.data = {}
        else:
            self.data = self.db.get('docker_opts')

    def __save(self):
        self.db.set('docker_opts', self.data)

    def add(self, key, value):
        ''' Adds data to the map of values for the DockerOpts file.
        Supports single values, or "multiopt variables"

        eg:
        opts.add('label', 'foo')
        opts.add('label', 'foo, bar, baz')
        '''
        values = [x.strip() for x in value.split(',')]
        if key in self.data:
            item_data = self.data[key]
            for c in values:
                c = c.strip()
                if not c in item_data:
                    item_data.append(c)
            self.data[key] = item_data
        else:
            self.data[key] = values

        self.__save()

    def remove(self, key, value):
        ''' Remove a flag value from the DockerOpts manager
        Assuming the data is currently {'foo': ['bar', 'baz']}

        d.remove('foo', 'bar')
        > {'foo': ['baz']}
        '''
        self.data[key].remove(value)
        self.__save()

    def to_s(self):
        ''' Render the flags to a single string, prepared for the Docker
        Defaults file. Typically in /etc/default/docker

        d.to_s()
        > "--foo=bar --foo=baz"
        '''
        flags = []
        for key in self.data:
            for item in self.data[key]:
                flags.append("--{}={}".format(key, item))
        return ' '.join(flags)



import os

from contextlib import contextmanager
from shlex import split
from subprocess import check_output
from workspace import Workspace

# Wrapper and convenience methods for charming w/ docker compose in python
class Compose:
    def __init__(self, workspace, strict=True):
        '''
        Object to manage working with Docker-Compose on the CLI. exposes
        a natural language for performing common tasks with docker in
        juju charms.

        @param workspace - Define the CWD for docker-compose execution

        @param strict - Enable/disable workspace validation
        '''
        self.workspace = Workspace(workspace)
        if strict:
            self.workspace.validate()

    def up(self, service=None):
        '''
        Convenience method that wraps `docker-compose up`

        usage: c.up('nginx')  to start the 'nginx' service from the
        defined `docker-compose.yml` as a daemon
        '''
        if service:
            cmd = "docker-compose up -d {}".format(service)
        else:
            cmd = "docker-compose up -d"
        self.run(cmd)

    def kill(self, service=None):
        '''
        Convenience method that wraps `docker-compose kill`

        usage: c.kill('nginx')  to kill the 'nginx' service from the
        defined `docker-compose.yml`
        '''
        if service:
            cmd = "docker-compose kill {}".format(service)
        else:
            cmd = "docker-compose kill"
        self.run(cmd)

    def run(self, cmd):
        '''
        chdir sets working context on the workspace

        @param: cmd - String of the command to run. eg: echo "hello world"
        the string is passed through shlex.parse() for convenience.

        returns STDOUT of command execution

        usage: c.run('docker-compose ps')
        '''
        with chdir("{}".format(self.workspace)):
            out = check_output(split(cmd))
            return out


# This is helpful for setting working directory context
@contextmanager
def chdir(path):
    '''Change the current working directory to a different directory to run
    commands and return to the previous directory after the command is done.'''
    old_dir = os.getcwd()
    os.chdir(path)
    yield
    os.chdir(old_dir)
